/*===========================================================================
SCRNWIN.C - Confines DOS STDIO to specific screen area by re-defining
            bios screen parameters. (Vertical constraint only.)

NOTES:  This alternate method of constraining screen output to a given area
of the screen has the advantage of EXTREMELY SMALL CODE SIZE.  The "guts" of
this consist of changing ONLY 4 BYTES of data in system memory.  Another
advantage of this method is that "paged" display such as DIR/P, MEM/C, or
commands using dos pipes (|MORE) work properly for the window size.  The
only disadvantage is no HORIZONTAL constraint is provided.  The window's
height is adjustable, but it will always be full screen width.

Two _far pointers are initialized to system physical addresses 0:044Eh and
0:0484h.  These addresses contain the video memory offset of video page 1
and the number of screen display rows respectively.  The dos STDIO routines
refer to these values for screen output location in text mode.

For both vertical AND horizontal control, see the companion file STDOWIN.C.
If both vertical and horizontal control PLUS PAGING are needed, experiment
by combining the functions from BOTH files.

This source code is hereby declared PUBLIC DOMAIN.
===========================================================================*/

#include <process.h>    /*needed for demo only*/
#include <stdio.h>      /*needed for demo only*/

void setoswindow(unsigned int top, unsigned int bot);
void resetwin(void);

unsigned int _far *p_vpbeg=(unsigned int _far *)0x0000044e;
unsigned int _far *p_rows=(unsigned int _far *)0x00000484;

unsigned int savevp, saverows;

void main() /*demo*/
{
system("cls");
printf("type EXIT to restore normal window");

setoswindow(8,16);      /*set a window of top=row 8, bottom=row 16*/
system("command.com");  /*shell to dos*/
resetwin();             /*restore original screen parameters*/

system("cls");
}

void setoswindow(unsigned int top, unsigned int bot)
	{
    savevp=*p_vpbeg;        /*save system defaults*/
	saverows=*p_rows;

    *p_rows=(bot-top);      /*set # of rows*/
    *p_vpbeg=(top*160)-160; /*set page1 offset (assumes 80 cols)*/
	}

void resetwin(void)     /*restore defaults*/
	{
	*p_rows=saverows;
	*p_vpbeg=savevp;
	}
